==> dillo-0.2.2.tar.gz: dillo-0.2.2/src/dillo.c <==
/*
 * Dillo web browser
 *
 * Copyright 1997 Raph Levien <raph@acm.org>
 * Copyright 1999, 2000 Jorge Arellano Cid <jcid@inf.utfsm.cl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <gtk/gtk.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>

#include "dillo.h"
#include "web.h"
#include "misc.h"
#include "cache.h"
#include "dw_gtk_statuslabel.h"
#include "browser.h"
#include "menu.h"
#include "cache.h"
#include "browser.h"
#include "nav.h"
#include "bookmark.h"
#include "dicache.h"
#include "dns.h"
#include "IO/Url.h"
#include "prefs.h"
#include "interface.h"

/* contains the browser_window structure */
#include "browser.h"

#include "dw.h"
#include "html.h"

/*
 * Forward declarations
 */
void Dillo_check_home_dir(char *dir);
gint Dillo_check_splash_screen(gchar *file);



/*
 * ******************************** MAIN *************************************
 */
gint main(int argc, char *argv[])
{
   gchar *file;
   GString *start_url;
   BrowserWindow *bw;


   /* Ignore any broken pipes: we used to get a lot */
   signal(SIGPIPE, SIG_IGN);

   gtk_true();
   gtk_init(&argc, &argv);

   /* check that ~/.dillo exists, create it if it doesn't */
   file = a_Misc_prepend_user_home(".dillo");
   Dillo_check_home_dir(file);
   g_free(file);

   a_Prefs_init();
   a_Dns_init();
   a_Url_init();
   a_Mime_init();
   a_Dicache_init();
   a_Interface_init();
   a_Dw_image_init();

   /* a_Nav_init() has been moved into this call because it needs to be
    * initialized with the new browser_window structure */
   bw = a_Interface_new_browser_window();

   a_Bookmarks_init();

   /* Send dillo startup screen */
   file = a_Misc_prepend_user_home(".dillo/splash022z.html");
   Dillo_check_splash_screen(file);
   start_url = g_string_new("file:");
   g_string_append(start_url, file);
   a_Nav_push(bw, start_url->str);
   g_free(file);
   g_string_free(start_url, TRUE);

   if (argc == 2) {
      if (a_Url_is_absolute(argv[1])) {
         a_Nav_push(bw, argv[1]);
      } else {
         /* CP: Would be wise to verify that URLs stay within those bounds */
         char url[1024];

         if (strlen(argv[1]) + 6 <= sizeof(url)) {
            /* CP: Just blithely assume it's a file URL?
             * Need more sophisticated heuristics. If there is a TLD, then 
             * proto = HTTP, for instance. */
            /* CP: Would also be wise to postpend '/' on dirs and domain 
             * names. */
            sprintf(url, "file:%s", argv[1]);
            a_Nav_push(bw, url);
         }
      }
   }
   gtk_main();

   /* 
    * Memory deallocating routines
    * (This can be left to the OS, but we'll do it, with a view to test
    *  and fix our memory management)
    */
   a_Cache_freeall();
   a_Dicache_freeall();
   a_Http_freeall();
   a_Dns_freeall();
   a_Prefs_freeall();

   printf("Dillo: normal exit!\n");
   return 0;
}

/*
 * Checks if '~/.dillo' directory exists.
 * If not, attempt to create it.
 */
void Dillo_check_home_dir(char *dir)
{
   struct stat st;

   if (stat(dir, &st) < 0) {
      /* directory does not exist, create */
      if ((mkdir(dir, 0700) < 0)) {
         g_print("Dillo: error creating directory %s: %s\n",
                 dir, strerror(errno));
         return;
      }
   }
}

/*
 * HTML text for startup screen
 */
static char *splash=
"<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">\n\
<html>\n\
<head>\n\
 <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n\
</head>\n\
<hr>\n\
 <h1></a>Dillo project<br>\n\
         <h3>Version 0.2.2 Developer's release</h3></h1>\n\
<hr>\n\
<h4> License </h4>\n\
<p>\n\
    This program is free software; you can redistribute it and/or modify\n\
    it under the terms of the GNU General Public License as published by\n\
    the Free Software Foundation; either version 2 of the License, or\n\
    (at your option) any later version.\n\
<p>\n\
    This program is distributed in the hope that it will be useful,\n\
    but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
    GNU General Public License for more details.\n\
\n\
<hr>\n\
<h4>NOTES:</h4>\n\
<p> \n\
<ul>\n\
<li> Dillo has expose problems: sometimes the page is there but you'll\n\
not be able to see it, or it will render misplaced. The trick\n\
is to pick the URL-handlebox (the small area to the left of NEW-button) click\n\
and drag it a bit, and then to place it back.</li>\n\
<li> Frame-based pages will not render. You can note them cause the progress\n\
bar will show content, but no rendering will be done. Press Ctrl-V (view\n\
source) to get sure. </li>\n\
<li> Dillo behaves very nice when browsing local files, images and HTML.\n\
It's also very good for Internet searching (try www.google.com). Test it!</li>\n\
<li> There's a dillorc file within the tarball; copy it to ~/.dillo/ directory.\n\
If you set force_my_colors=YES, slashdot will get readable!</li>
<li> There's developer documentation in the /doc dir within the tarball, and\n\
you can find directions on everything else at the home page. </li>\n\
<li> Tables and frames are not supported yet.</a></li>\n\
<li> This release is intended <strong>for developers only.</strong></a></li>\n\
</ul>\n\
<hr>\n\
</body>\n\
</html>\n\
";

/*
 * Check if splash screen (HTML file) is saved in .dillo/
 * (Save if not)
 */
gint Dillo_check_splash_screen(gchar *file)
{
   struct stat st;
   FILE *stream;

   if ( stat(file, &st) < 0 ) {
      /* splash screen not saved, create it */
      if ( (stream = fopen(file, "w")) != NULL){
         fprintf(stream, splash);
         fclose(stream);
      }
   }
   return 0;
}

==> dillo_0.6.4.orig.tar.gz: dillo-0.6.4/src/dillo.c <==
/*
 * Dillo web browser
 *
 * Copyright 1997 Raph Levien <raph@acm.org>
 * Copyright 1999, 2000, 2001 Jorge Arellano Cid <jcid@inf.utfsm.cl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <gtk/gtk.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>

#include "dillo.h"
#include "misc.h"
#include "dw_gtk_statuslabel.h"
#include "nav.h"
#include "history.h"
#include "bookmark.h"
#include "dicache.h"
#include "dns.h"
#include "IO/Url.h"
#include "IO/mime.h"
#include "prefs.h"
#include "interface.h"
#include "dw.h"
#include "dw_widget.h"


/*
 * Forward declarations
 */
static void Dillo_check_home_dir(char *dir);



/*
 * ******************************** MAIN *************************************
 */
gint main(int argc, char *argv[])
{
   gchar *file;
   DilloUrl *start_url;
   BrowserWindow *bw;

   /* This lets threads in the file module end peacefully when aborted
    * todo: implement a cleaner mechanism (in file.c) */
   signal(SIGPIPE, SIG_IGN);

   g_print("Setting locale to %s\n", gtk_set_locale());
   gtk_init(&argc, &argv);
   gdk_rgb_init();

   /* check that ~/.dillo exists, create it if it doesn't */
   file = a_Misc_prepend_user_home(".dillo");
   Dillo_check_home_dir(file);
   g_free(file);

   a_Prefs_init();
   a_Dns_init();
   a_Http_init();
   a_Mime_init();
   a_Dicache_init();
   a_Interface_init();
   a_Dw_init();

   /* a_Nav_init() has been moved into this call because it needs to be
    * initialized with the new browser_window structure */
   bw = a_Interface_browser_window_new(prefs.width, prefs.height);

   a_Bookmarks_init();

   /* Send dillo startup screen */
   start_url = a_Url_new("splash", "about:", 0, 0);
   a_Nav_push(bw, start_url);
   a_Url_free(start_url);

   if (argc == 2) {
      if (access(argv[1], F_OK) == 0) {
         GString *UrlStr = g_string_sized_new(128);

         if (argv[1][0] == '/') {
            g_string_sprintf(UrlStr, "file:%s", argv[1]);
         } else {
            g_string_sprintf(UrlStr, "file:%s", g_get_current_dir());
            if (UrlStr->str[UrlStr->len - 1] != '/')
               g_string_append(UrlStr, "/");
            g_string_append(UrlStr, argv[1]);
         }
         start_url = a_Url_new(UrlStr->str, NULL, 0, 0);
         g_string_free(UrlStr, TRUE);
      } else {
         start_url = a_Url_new(argv[1], NULL, 0, 0);
      }
      a_Nav_push(bw, start_url);
      a_Url_free(start_url);
   }


   gtk_main();

   /*
    * Memory deallocating routines
    * (This can be left to the OS, but we'll do it, with a view to test
    *  and fix our memory management)
    */
   a_Cache_freeall();
   a_Dicache_freeall();
   a_Http_freeall();
   a_Dns_freeall();
   a_Prefs_freeall();
   a_Dw_freeall();
   a_History_free();

   printf("Dillo: normal exit!\n");
   return 0;
}

/*
 * Checks if '~/.dillo' directory exists.
 * If not, attempt to create it.
 */
static void Dillo_check_home_dir(char *dir)
{
   struct stat st;

   if ( stat(dir, &st) == -1 ) {
      if ( errno == ENOENT && mkdir(dir, 0700) < 0 ) {
         g_print("Dillo: error creating directory %s: %s\n",
                 dir, strerror(errno));
      } else
         g_print("Dillo: error reading %s: %s\n", dir, strerror(errno));
   }
}
